@@ -21,27 +21,58 @@ This module implements Quadratic Majorize-Minimize optimization algorithms.
 
 """
 
-# pylint: disable=bad-continuation
-
 import abc
-import itertools as it
+import collections.abc
 import time
-from functools import reduce
+from functools import reduce, wraps
 from operator import iadd
-from typing import Callable, List, Sequence, Tuple, Union
+from typing import (Callable, List, MutableSequence, Optional, Sequence, Tuple,
+                    Union)
 
 import numpy as np  # type: ignore
 import numpy.linalg as la  # type: ignore
 from numpy import ndarray as array
 
+__author__ = "François Orieux"
+__copyright__ = "2021, François Orieux <francois.orieux@universite-paris-saclay.fr>"
+__credits__ = ["François Orieux"]
+__license__ = "GPL-3.0-or-later"
+__version__ = "0.9.0"
+__maintainer__ = "François Orieux"
+__email__ = "francois.orieux@universite-paris-saclay.fr"
+__status__ = "beta"
+__url__ = "https://github.com/forieux/qmm/"
+
+
 ArrOrSeq = Union[array, Sequence[array]]
 
+__all__ = [
+    "OptimizeResult",
+    "mmmg",
+    "mmcg",
+    "lcg",
+    "BaseObjective",
+    "MixedObjective",
+    "Objective",
+    "QuadObjective",
+    "Vmin",
+    "Vmax",
+    "Loss",
+    "Square",
+    "Huber",
+    "Hyperbolic",
+    "HebertLeahy",
+    "GemanMcClure",
+    "TruncSquareApprox",
+    "vectorize",
+]
+
 
 class OptimizeResult(dict):
     """Represents the optimization result.
 
     x: array
-        The solution of the optimization, with same shape than `init`.
+        The solution of the optimization, with same shape than `x0`.
     success: bool
         Whether or not the optimizer exited successfully.
     status: int
@@ -51,48 +82,50 @@ class OptimizeResult(dict):
         Description of the cause of the termination.
     nit: int
         Number of iterations performed by the optimizer.
-    grad_norm: list of float
-        The gradient norm at each iteration
     diff: list of float
         The value of ||x^(k+1) - x^(k)||² at each iteration
     time: list of float
         The time at each iteration, starting at 0, in seconds.
     fun: float
         The value of the objective function.
-    jac: array
-        The gradient of the objective function.
     objv_val: list of float
         The objective value at each iteration
+    jac: array
+        The gradient of the objective function.
+    grad_norm: list of float
+        The gradient norm at each iteration
 
     Notes
     -----
-    :class:`OptimizeResult` mime `OptimizeResult` of scipy for compatibility.
+    :class:`OptimizeResult` mimes `OptimizeResult` of scipy for compatibility.
 
     """
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.maxcv = 0
-        self.nfev = 0
-        self.nhev = 0
         self.jac = None
-        self.jav = None
-        self.hess = None
-        self.hess_inv = None
         self.success = False
         self.status = 99
-        self.message = "Not applicable"
+        self.message = "NA"
         self.njev = 0
         self.nit = 0
         self.grad_norm = []
         self.diff = []
         self.time = []
         self.objv_val = []
-        self.x = None
+        self.x = None  # pylint: disable=invalid-name
 
     @property
     def fun(self):
-        return self.objv_val[-1]
+        """Last objective value"""
+        if len(self.objv_val) != 0:
+            return self.objv_val[-1]
+        else:
+            return None
+
+    @fun.setter
+    def fun(self, value):
+        self.objv_val.append(value)
 
     def __getattr__(self, name):
         if name == "fun":
@@ -102,6 +135,8 @@ class OptimizeResult(dict):
         raise AttributeError("No such attribute: " + name)
 
     def __setattr__(self, name, value):
+        if name == "fun":
+            return self["objv_val"].append(value)
         self[name] = value
 
     def __delattr__(self, name):
@@ -110,14 +145,26 @@ class OptimizeResult(dict):
         else:
             raise AttributeError("No such attribute: " + name)
 
+    def __repr__(self):
+        if self.keys():
+            m = max(map(len, list(self.keys()))) + 1
+            return "\n".join(
+                [k.rjust(m) + ": " + str(v) for k, v in sorted(self.items())]
+            )
+        return self.__class__.__name__ + "()"
+
+    def __dir__(self):
+        return list(self.keys())
+
 
 def mmmg(
     objv_list: Sequence["BaseObjective"],
-    init: array,
+    x0: array,
     tol: float = 1e-4,
     max_iter: int = 500,
-    callback: Callable[[OptimizeResult], None] = None,
-    calc_objv: bool = False,
+    precond: Optional[Callable[[array], array]] = None,
+    callback: Optional[Callable[[OptimizeResult], None]] = None,
+    calc_fun: bool = False,
 ) -> OptimizeResult:
     r"""The Majorize-Minimize Memory Gradient (`3mg`) algorithm.
 
@@ -128,19 +175,22 @@ def mmmg(
     Parameters
     ----------
     objv_list : list of `BaseObjective`
-        A list of :class:`BaseObjective` objects that each represent a `μ ψ(V·x - ω)`.
-        The objectives are implicitly summed.
-    init : array
+        A list of :class:`BaseObjective` objects that each represents
+        a `μ ψ(V·x - ω)`. The objectives are summed.
+    x0 : array
         The initial point.
     tol : float, optional
         The stopping tolerance. The algorithm is stopped when the gradient norm
-        is inferior to `init.size * tol`.
+        is inferior to `x0.size * tol`.
     max_iter : int, optional
         The maximum number of iterations.
+    precond : callable, optional
+        A callable that must implement a preconditioner, that is `P·x`. Must
+        be a callable with a unique input parameter `x` and unique output.
     callback : callable, optional
         A function that receive the `OptimizeResult` at the end of each
         iteration.
-    calc_objv: boolean, optional
+    calc_fun: boolean, optional
         If True, objective function is computed at each iteration with low
         overhead. False by default. Not used by the algorithm.
 
@@ -155,88 +205,92 @@ def mmmg(
        Trans. on Image Process., vol. 20, no. 6, pp. 1517–1528, Jun. 2011, doi:
        10.1109/TIP.2010.2103083.
     """
+    if precond is None:
+        precond = lambda x: x
     res = OptimizeResult()
     previous_flag = []
     for objv in objv_list:
-        previous_flag.append(objv.calc_objv)
-        objv.calc_objv = calc_objv
+        previous_flag.append(objv.calc_fun)
+        objv.calc_fun = calc_fun
 
-    res["x"] = init.copy().reshape((-1, 1))
+    res.x = x0.copy().reshape((-1, 1))
 
     # The first previous moves are initialized with 0 array. Consequently, the
     # first iterations implementation can be improved, at the cost of if
     # statement.
-    move = np.zeros_like(res["x"])
+    move = np.zeros_like(res.x)
     op_directions = [
-        np.tile(_vect(objv.operator, move, init.shape), 2) for objv in objv_list
+        np.tile(vect(objv.operator, move, x0.shape), 2) for objv in objv_list
     ]
     step = np.ones((2, 1))
 
-    res["time"].append(time.time())
+    res.time.append(time.time())
 
     for iteration in range(max_iter):
         # Vectorized gradient
-        grad = _gradient(objv_list, res["x"], init.shape)
-        res["grad_norm"].append(la.norm(grad))
-        res["jac"] = grad.reshape(init.shape)
-        res["objv_val"].append(_lastv(objv_list))
+        grad = _gradient(objv_list, res.x, x0.shape)
+        res.grad_norm.append(la.norm(grad))
+        res.jac = grad.reshape(x0.shape)
+        res.fun = lastgv(objv_list)
 
         # Stopping test
-        if res["grad_norm"][-1] < init.size * tol:
-            res["success"] = True
-            res["status"] = 0
+        if res.grad_norm[-1] < x0.size * tol:
+            res.success = True
+            res.status = 0
             break
 
         # Memory gradient directions
-        directions = np.c_[-grad, move]
+        new_dir = vect(precond, grad, x0.shape)
+        directions = np.c_[-new_dir, move]
 
         # Step by Majorize-Minimize
         op_directions = [
-            np.c_[_vect(objv.operator, grad, init.shape), i_op_dir @ step]
+            np.c_[vect(objv.operator, new_dir, x0.shape), i_op_dir @ step]
             for objv, i_op_dir in zip(objv_list, op_directions)
         ]
         step = -la.pinv(
             sum(
-                objv.norm_mat_major(i_op_dir, res["x"].reshape(init.shape))
+                objv.norm_mat_major(i_op_dir, res.x.reshape(x0.shape))
                 for objv, i_op_dir in zip(objv_list, op_directions)
             )
         ) @ (directions.T @ grad)
         move = directions @ step
 
         # update
-        res["x"] += move
+        res.x += move
 
-        res["diff"].append(np.sum(move) ** 2)
-        res["time"].append(time.time())
+        res.diff.append(np.sum(move) ** 2)
+        res.time.append(time.time())
 
         if callback is not None:
             callback(res)
 
     if res.status == 0:
-        res["message"] = "Stopping conditions reached"
+        res.message = "Stopping conditions reached."
     else:
-        res["success"] = False
-        res["status"] = 1
-        res["message"] = "Maximum number of iteration reached"
-    res["x"] = res["x"].reshape(init.shape)
-    res["njev"] = iteration + 1
-    res["nit"] = iteration + 1
-    res["time"] = list(np.asarray(res.time) - res.time[0])
+        res.success = False
+        res.status = 1
+        res.message = "Maximum number of iterations has been exceeded."
+        del res.time[-1]
+    res.x = np.reshape(res.x, x0.shape)
+    res.njev = iteration + 1
+    res.nit = iteration + 1
+    res.time = list(np.asarray(res.time) - res.time[0])
 
     for objv, flag in zip(objv_list, previous_flag):
-        objv.calc_objv = flag
+        objv.calc_fun = flag
 
     return res
 
 
 def mmcg(
     objv_list: Sequence["BaseObjective"],
-    init: array,
-    precond: Callable[[array], array] = None,
+    x0: array,
     tol: float = 1e-4,
     max_iter: int = 500,
-    callback: Callable[[OptimizeResult], None] = None,
-    calc_objv: bool = False,
+    precond: Optional[Callable[[array], array]] = None,
+    callback: Optional[Callable[[OptimizeResult], None]] = None,
+    calc_fun: bool = False,
 ) -> OptimizeResult:
     """The Majorize-Minimize Conjugate Gradient (MM-CG) algorithm.
 
@@ -247,22 +301,22 @@ def mmcg(
     Parameters
     ----------
     objv_list : list of `BaseObjective`
-        A list of :class:`BaseObjective` objects that each represent a `μ ψ(V·x - ω)`.
-        The objectives are implicitly summed.
-    init : ndarray
+        A list of :class:`BaseObjective` objects that each represents
+        a `μ ψ(V·x - ω)`. The objectives are summed.
+    x0 : ndarray
         The initial point.
-    precond : callable, optional
-        A callable that must implement a preconditioner, that is `M⁻¹·x`. Must
-        be a callable with a unique input parameter `x` and unique output.
     tol : float, optional
         The stopping tolerance. The algorithm is stopped when the gradient norm
-        is inferior to `init.size * tol`.
+        is inferior to `x0.size * tol`.
     max_iter : int, optional
         The maximum number of iterations.
+    precond : callable, optional
+        A callable that must implement a preconditioner, that is `P·x`. Must
+        be a callable with a unique input parameter `x` and unique output.
     callback : callable, optional
         A function that receive the `OptimizeResult` at the end of each
         iteration.
-    calc_objv: boolean, optional
+    calc_fun: boolean, optional
         If True, objective function is computed at each iteration with low
         overhead. False by default. Not used by the algorithm.
 
@@ -280,49 +334,47 @@ def mmcg(
     res = OptimizeResult()
     previous_flag = []
     for objv in objv_list:
-        previous_flag.append(objv.calc_objv)
-        objv.calc_objv = calc_objv
+        previous_flag.append(objv.calc_fun)
+        objv.calc_fun = calc_fun
 
-    res["x"] = init.copy().reshape((-1, 1))
+    res.x = x0.copy().reshape((-1, 1))
 
-    residual = -_gradient(objv_list, res["x"], init.shape)
-    sec = _vect(precond, residual, init.shape)
+    residual = -_gradient(objv_list, res.x, x0.shape)
+    sec = vect(precond, residual, x0.shape)
     direction = sec
     delta = residual.T @ direction
 
-    res["time"].append(time.time())
+    res.time.append(time.time())
 
     for iteration in range(max_iter):
         # Stop test
-        res["grad_norm"].append(la.norm(residual))
-        if res["grad_norm"][-1] < init.size * tol:
+        res.grad_norm.append(la.norm(residual))
+        if res.grad_norm[-1] < x0.size * tol:
             break
 
         # update
-        op_direction = [
-            _vect(objv.operator, direction, init.shape) for objv in objv_list
-        ]
+        op_direction = [vect(objv.operator, direction, x0.shape) for objv in objv_list]
 
         step = direction.T @ residual
         step = step / sum(
-            objv.norm_mat_major(i_op_dir, res["x"].reshape(init.shape))
+            objv.norm_mat_major(i_op_dir, res.x.reshape(x0.shape))
             for objv, i_op_dir in zip(objv_list, op_direction)
         )
 
-        res["x"] += step * direction
+        res.x += step * direction
 
-        res["diff"].append(np.sum(step * direction) ** 2)
-        res["time"].append(time.time())
+        res.diff.append(np.sum(step * direction) ** 2)
+        res.time.append(time.time())
 
         # Gradient
-        residual = -_gradient(objv_list, res["x"], init.shape)
-        res["jac"] = -residual.reshape(init.shape)
-        res["objv_val"].append(_lastv(objv_list))
+        residual = -_gradient(objv_list, res.x, x0.shape)
+        res.jac = -residual.reshape(x0.shape)
+        res.fun = lastgv(objv_list)
 
         # Conjugate direction. No reset is done, see Shewchuck.
         delta_old = delta
         delta_mid = residual.T @ sec
-        sec = _vect(precond, residual, init.shape)
+        sec = vect(precond, residual, x0.shape)
         delta = residual.T @ sec
         if (delta - delta_mid) / delta_old >= 0:
             direction = sec + (delta - delta_mid) / delta_old * direction
@@ -333,30 +385,30 @@ def mmcg(
             callback(res)
 
     if res.status == 0:
-        res["message"] = "Stopping conditions reached"
+        res.message = "Stopping conditions reached."
     else:
-        res["success"] = False
-        res["status"] = 1
-        res["message"] = "Maximum number of iteration reached"
-    res["x"] = res["x"].reshape(init.shape)
-    res["njev"] = iteration + 1
-    res["nit"] = iteration + 1
-    res["time"] = list(np.asarray(res.time) - res.time[0])
+        res.success = False
+        res.status = 1
+        res.message = "Maximum number of iterations has been exceeded."
+    res.x = np.reshape(res.x, x0.shape)
+    res.njev = iteration + 1
+    res.nit = iteration + 1
+    res.time = list(np.asarray(res.time) - res.time[0])
 
     for objv, flag in zip(objv_list, previous_flag):
-        objv.calc_objv = flag
+        objv.calc_fun = flag
 
     return res
 
 
 def lcg(
     objv_list: Sequence["QuadObjective"],
-    init: array,
-    precond: Callable[[array], array] = None,
+    x0: array,
     tol: float = 1e-4,
     max_iter: int = 500,
-    callback: Callable[[OptimizeResult], None] = None,
-    calc_objv: bool = False,
+    precond: Optional[Callable[[array], array]] = None,
+    callback: Optional[Callable[[OptimizeResult], None]] = None,
+    calc_fun: bool = False,
 ) -> OptimizeResult:
     """Linear Conjugate Gradient (CG) algorithm.
 
@@ -365,22 +417,22 @@ def lcg(
     Parameters
     ----------
     objv_list : list of `QuadObjective`
-        A list of :class:`QuadObjective` objects that each represent a `½ μ
-        ||V·x - ω||²`. The objectives are implicitly summed.
-    init : ndarray
+        A list of :class:`QuadObjective` objects that each represents
+        a `½ μ ||V·x - ω||²`. The objectives are summed.
+    x0 : ndarray
         The initial point.
     precond : callable, optional
-        A callable that must implement a preconditioner, that is `M⁻¹·x`. Must
+        A callable that must implement a preconditioner, that is `P·x`. Must
         be a callable with a unique input parameter `x` and unique output.
     tol : float, optional
         The stopping tolerance. The algorithm is stopped when the gradient norm
-        is inferior to `init.size * tol`.
+        is inferior to `x0.size * tol`.
     max_iter : int, optional
         The maximum number of iterations.
     callback : callable, optional
         A function that receive the `OptimizeResult` at the end of each
         iteration.
-    calc_objv: boolean, optional
+    calc_fun: boolean, optional
         If True, objective function is computed at each iteration with low
         overhead. False by default. Not used by the algorithm.
 
@@ -393,95 +445,109 @@ def lcg(
     if precond is None:
         precond = lambda x: x
     res = OptimizeResult()
-    previous_flag = []
-    for objv in objv_list:
-        previous_flag.append(objv.calc_objv)
-        objv.calc_objv = calc_objv
 
-    res["x"] = init.copy().reshape((-1, 1))
+    res.x = x0.copy().reshape((-1, 1))
 
-    second_term = np.reshape(reduce(iadd, (c.data_t for c in objv_list)), (-1, 1))
+    second_term = np.reshape(reduce(iadd, (c.hdata_t for c in objv_list)), (-1, 1))
+    constant = reduce(iadd, (c.constant for c in objv_list))
 
     def hessian(arr):
-        return reduce(iadd, (_vect(c.hessp, arr, init.shape) for c in objv_list))
+        return reduce(iadd, (vect(c.hessp, arr, x0.shape) for c in objv_list))
 
-    # Gradient at current init
-    residual = second_term - hessian(res["x"])
-    direction = _vect(precond, residual, init.shape)
+    def value_residual(arr, residual):
+        return (np.sum(arr * (-second_term - residual)) + constant) / 2
 
-    res["grad_norm"].append(np.sum(np.real(np.conj(residual) * direction)))
-    res["time"].append(time.time())
+    # Gradient at current x0
+    residual = second_term - hessian(res.x)
+    direction = vect(precond, residual, x0.shape)
+
+    res.grad_norm.append(np.sum(np.real(np.conj(residual) * direction)))
+    res.time.append(time.time())
 
     for iteration in range(max_iter):
-        hess_dir = hessian(direction)
+        hessp = hessian(direction)
+        if calc_fun:
+            res.fun = value_residual(res.x, residual)
+
         # s = rᵀr / dᵀAd
         # Optimal step
-        step = res.grad_norm[-1] / np.sum(np.real(np.conj(direction) * hess_dir))
+        step = res.grad_norm[-1] / np.sum(np.real(np.conj(direction) * hessp))
 
         # Descent x^(i+1) = x^(i) + s*d
-        res["x"] += step * direction
+        res.x += step * direction
 
         # r^(i+1) = r^(i) - s * A·d
         if iteration % 50 == 0:
-            residual = second_term - hessian(res["x"])
+            residual = second_term - hessian(res.x)
         else:
-            residual -= step * hess_dir
-        res["jac"] = -residual.reshape(init.shape)
-        res["objv_val"].append(_lastv(objv_list))
+            residual -= step * hessp
+        res.jac = -residual.reshape(x0.shape)
 
         # Conjugate direction with preconditionner
-        secant = _vect(precond, residual, init.shape)
-        res["grad_norm"].append(np.sum(np.real(np.conj(residual) * secant)))
-        direction = secant + (res["grad_norm"][-1] / res["grad_norm"][-2]) * direction
+        secant = vect(precond, residual, x0.shape)
+        res.grad_norm.append(np.sum(np.real(np.conj(residual) * secant)))
+        direction = secant + (res.grad_norm[-1] / res.grad_norm[-2]) * direction
 
-        res["diff"].append(np.sum(step * direction) ** 2)
-        res["time"].append(time.time())
+        res.diff.append(np.sum(step * direction) ** 2)
+        res.time.append(time.time())
 
         # Stopping condition
-        if np.sqrt(res.grad_norm[-1]) < init.size * tol:
-            res["success"] = True
-            res["status"] = 0
+        if np.sqrt(res.grad_norm[-1]) < x0.size * tol:
+            res.success = True
+            res.status = 0
             break
 
         if callback is not None:
             callback(res)
 
     if res.status == 0:
-        res["message"] = "Stopping conditions reached"
+        res.message = "Stopping conditions reached."
     else:
-        res["success"] = False
-        res["status"] = 1
-        res["message"] = "Maximum number of iteration reached"
-    res["x"] = res.x.reshape(init.shape)
-    res["njev"] = iteration + 1
-    res["nit"] = iteration + 1
-    res["grad_norm"] = list(np.sqrt(res.grad_norm))
-    res["time"] = list(np.asarray(res.time) - res.time[0])
-
-    for objv, flag in zip(objv_list, previous_flag):
-        objv.calc_objv = flag
+        res.success = False
+        res.status = 1
+        res.message = "Maximum number of iterations has been exceeded."
+    res.x = np.reshape(res.x, x0.shape)
+    res.njev = iteration + 1
+    res.nit = iteration + 1
+    res.grad_norm = list(np.sqrt(res.grad_norm))
+    res.time = list(np.asarray(res.time) - res.time[0])
 
     return res
 
 
 # Vectorized call
-def _vect(func: Callable[[array], array], point: array, shape: Tuple) -> array:
+def vect(func: Callable[[array], array], point: array, shape: Tuple) -> array:
     """Call func with point reshaped as shape and return vectorized output"""
     return np.reshape(func(np.reshape(point, shape)), (-1, 1))
 
 
+def vectorize(shape: Tuple) -> Callable:
+    """Return a decorator to vectorize input and output given `shape`"""
+
+    def decorator(func: Callable[[array], array]) -> Callable[[array], array]:
+        """A decorator to vectorize input and output"""
+
+        @wraps(func)
+        def wrapper(arr: array) -> array:
+            return np.reshape(func(np.reshape(arr, shape)), (-1,))
+
+        return wrapper
+
+    return decorator
+
+
 # Vectorized gradient
 def _gradient(
     objv_list: Sequence["BaseObjective"], point: array, shape: Tuple
 ) -> array:
     """Compute sum of gradient with vectorized parameters and return"""
     # The use of reduce and iadd do an more efficient numpy inplace sum
-    return reduce(iadd, (_vect(c.gradient, point, shape) for c in objv_list))
+    return reduce(iadd, (vect(o.gradient, point, shape) for o in objv_list))
 
 
-def _lastv(objv_list: Sequence["BaseObjective"]):
+def lastgv(objv_list: Sequence["BaseObjective"]):
     """Return the value of objective computed after gradient evaluation"""
-    return sum(getattr(objv, "lastv") for objv in objv_list)
+    return sum(objv.lastgv for objv in objv_list)
 
 
 class BaseObjective(abc.ABC):
@@ -491,27 +557,36 @@ class BaseObjective(abc.ABC):
         J(x) = \mu \Psi \left(V x - \omega \right)
 
     with :math:`\Psi(u) = \sum_i \varphi(u_i)`.
+
+    Attributes
+    ----------
+    calc_fun: boolean
+        If true, compute the objective value when gradient is computed and store
+        in `lastgv` attribute (False by default).
+    name: str
+        The name of the objective.
+    lastv: float
+        The last evaluated value of the objective (0 by default).
+    lastgv: float
+        The value of objective obtained during gradient computation (0 by default).
     """
 
-    def __init__(self):
-        self._lastv = -1
-        self.calc_objv = False
-
-    @property
-    def lastv(self):
-        """Return the value of objective after gradient computation."""
-        return self._lastv
-
-    @lastv.setter
-    def lastv(self, val):
-        """Return the value of objective after gradient computation."""
-        self._lastv = val
+    def __init__(self, name=""):
+        self.lastgv = 0
+        self.lastv = 0
+        self.calc_fun = False
+        self.name = name
 
     @abc.abstractmethod
     def operator(self, point: array) -> array:
         """Compute the output of `V·x`."""
         return NotImplemented
 
+    @abc.abstractmethod
+    def value(self, point: array) -> float:
+        """Compute the value at current point."""
+        return NotImplemented
+
     @abc.abstractmethod
     def gradient(self, point: array) -> array:
         """Compute the gradient at current point."""
@@ -540,6 +615,89 @@ class BaseObjective(abc.ABC):
         """
         return NotImplemented
 
+    def __add__(self, objv: "BaseObjective"):
+        return MixedObjective([self, objv])
+
+
+class MixedObjective(collections.abc.MutableSequence):
+    r"""Represents a mixed objective function
+
+    .. math::
+        J(x) = \sum_k \mu_k \Psi_k \left(V_k x - \omega_k \right)
+
+    This is a `Sequence` (or list-like) and instance of this class can be used
+    in optimization algorithms.
+
+    """
+
+    def __init__(self, objv_list: MutableSequence[BaseObjective]):
+        r"""A mixed objective function
+
+        `J(x) = ∑ₖ μₖ ψₖ(Vₖ·x - ωₖ)`.
+
+        Parameters
+        ----------
+        objv_list: list of `BaseObjective`
+
+        """
+        self._objv_list = objv_list
+
+    @property
+    def lastv(self):
+        """Return the value of objectives obtained during gradient computation."""
+        return sum(o.lastv for o in self._objv_list)
+
+    def __getitem__(self, key):
+        return self._objv_list.__getitem__(key)
+
+    def __setitem__(self, key, value):
+        return self._objv_list.__setitem__(key, value)
+
+    def __delitem__(self, key):
+        return self._objv_list.__delitem__(key)
+
+    def __len__(self):
+        return len(self._objv_list)
+
+    def insert(self, index, value):
+        return self._objv_list.insert(index, value)
+
+    def value(self, point: array) -> float:
+        """The value J(x)"""
+        return reduce(iadd, (o.value(point) for o in self._objv_list))
+
+    def gradient(self, point: array) -> array:
+        """The gradient ∇J(x)"""
+        return reduce(iadd, (o.gradient(point) for o in self._objv_list))
+
+    def __call__(self, point: array) -> float:
+        return self.value(point)
+
+    def __add__(self, objv: Union["BaseObjective", "MixedObjective"]):
+        if isinstance(objv, BaseObjective):
+            self._objv_list.append(objv)
+        elif isinstance(objv, MixedObjective):
+            print("here")
+            self._objv_list.extend(objv._objv_list)
+        else:
+            raise TypeError(
+                "unsupported operand type(s) for +: 'MixedObjective' and {}".format(
+                    type(objv)
+                )
+            )
+        return self
+
+    def __radd__(self, objv: "BaseObjective"):
+        if isinstance(objv, BaseObjective):
+            self._objv_list.append(objv)
+        else:
+            raise TypeError(
+                "unsupported operand type(s) for +: 'MixedObjective' and {}".format(
+                    type(objv)
+                )
+            )
+        return self
+
 
 class Objective(BaseObjective):
     r"""An objective function defined as
@@ -565,6 +723,7 @@ class Objective(BaseObjective):
         loss: "Loss",
         data: ArrOrSeq = None,
         hyper: float = 1,
+        name: str = "",
     ):
         """A objective function `μ ψ(V·x - ω)`.
 
@@ -580,6 +739,8 @@ class Objective(BaseObjective):
             The data vector `ω`.
         hyper: float, optional
             The hyperparameter `μ`.
+        name: str
+            The name of the objective.
 
         Notes
         -----
@@ -592,9 +753,9 @@ class Objective(BaseObjective):
         In that case, however, and for algorithm purpose, everything is
         internally stacked as a column vector and values are therefore copied.
         This is not efficient but flexible. Users are encouraged to do the
-        vectorization themselves and not use the list of array feature.
+        vectorization themselves and not use the "list of array" feature.
         """
-        super().__init__()
+        super().__init__(name=name)
         self._operator = operator
         self._adjoint = adjoint
 
@@ -637,7 +798,8 @@ class Objective(BaseObjective):
 
         Return `μ ψ(V·x - ω)`.
         """
-        return self.hyper * np.sum(self.loss(self.operator(point) - self.data))
+        self.lastv = self.hyper * np.sum(self.loss(self.operator(point) - self.data))
+        return self.lastv
 
     def gradient(self, point: array) -> array:
         """The gradient and value at given point
@@ -645,12 +807,14 @@ class Objective(BaseObjective):
         Return `μ Vᵀ·φ'(V·x - ω)`.
         """
         residual = self.operator(point) - self.data
-        if self.calc_objv:
-            self.lastv = self.hyper * np.sum(self.loss(residual))
+        if self.calc_fun:
+            self.lastgv = self.hyper * np.sum(self.loss(residual))
         return self.hyper * self.adjoint(self.loss.gradient(residual))
 
     def norm_mat_major(self, vecs: array, point: array) -> array:
-        matrix = vecs.T @ (self.gr_coeffs(point).reshape((-1, 1)) * vecs)
+        matrix = np.real(
+            np.conj(vecs.T) @ (self.gr_coeffs(point).reshape((-1, 1)) * vecs)
+        )
         return float(matrix) if matrix.size == 1 else matrix
 
     def gr_coeffs(self, point: array) -> array:
@@ -664,6 +828,9 @@ class Objective(BaseObjective):
     def __call__(self, point: array) -> float:
         return self.value(point)
 
+    def __repr__(self):
+        return f"{self.name} ({type(self).__name__}, loss: {type(self.loss).__name__}), hyper {self.hyper}, last eval {self.lastv}"
+
 
 class QuadObjective(Objective):
     r"""A quadratic objective function
@@ -671,10 +838,12 @@ class QuadObjective(Objective):
     .. math::
         :nowrap:
 
+        \begin{equation}
         \begin{aligned}
         J(x) & = \frac{1}{2} \mu \|V x - \omega\|_B^2 \\
-             & = \frac{1}{2} \mu (V x - \omega)^tB(V x - \omega) \\
+             & = \frac{1}{2} \mu (V x - \omega)^tB(V x - \omega)
         \end{aligned}
+        \end{equation}
 
     data : array
         The `data` array, or the vectorized list of array given at init.
@@ -691,7 +860,8 @@ class QuadObjective(Objective):
         hessp: Callable[[array], array] = None,
         data: array = None,
         hyper: float = 1,
-        metric: array = None,
+        invcovp: Callable[[array], array] = None,
+        name: str = "",
     ):
         """A quadratic objective `½ μ ||V·x - ω||²_B`
 
@@ -707,9 +877,11 @@ class QuadObjective(Objective):
             The data vector `ω`.
         hyper: float, optional
             The hyperparameter `μ`.
-        metric: array, optional
-            The **diagonal** of the metric matrix `B`. Equivalent to Identity if
-            not provided.
+        invcovp: callable, optional
+            A callable that apply the inverse covariance, or metric, `B`.
+            Equivalent to Identity if not provided.
+        name: str
+            The name of the objective.
 
         Notes
         -----
@@ -720,36 +892,33 @@ class QuadObjective(Objective):
         The variable `b = B·Vᵀ·ω` is computed at object creation.
 
         """
-        super().__init__(operator, adjoint, Square(), data=data, hyper=hyper)
-        self._metric = metric
+        super().__init__(operator, adjoint, Square(), data=data, hyper=hyper, name=name)
+
+        if invcovp is None:
+            self.invcovp = lambda x: x
+        else:
+            self.invcovp = invcovp
 
         if hessp is not None:
             self.hessp = lambda x: hyper * hessp(x)
         else:
-            self.hessp = lambda x: hyper * adjoint(self._metricp(operator(x)))
+            self.hessp = lambda x: hyper * adjoint(self.invcovp(operator(x)))
 
         if data is None:
-            self.data_t = 0
+            self.hdata_t = 0
             self.constant = 0  # c = μ ωᵀ·B·ω
         else:
-            self.data_t = hyper * self._metricp(adjoint(data))
-            self.constant = hyper * np.sum(data * self._metricp(data))  # c = μ ωᵀ·B·ω
-
-    def _metricp(self, arr: array) -> array:
-        if self._metric is None:
-            return arr
-        return self._metric * arr
+            self.hdata_t = hyper * self.invcovp(adjoint(data))
+            self.constant = hyper * np.sum(data * self.invcovp(data))  # c = μ ωᵀ·B·ω
 
     def value(self, point: array) -> float:
         """The value of the objective function at given point
 
         Return `½ μ ||V·x - ω||²_B`.
         """
-        return (
-            self.hyper
-            * np.sum(self._metricp((self.operator(point) - self.data) ** 2))
-            / 2
-        )
+        diff = self.operator(point) - self.data
+        self.lastv = self.hyper * np.sum(diff * self.invcovp(diff)) / 2
+        return self.lastv
 
     def gradient(self, point: array) -> array:
         """The gradient and value at given point
@@ -763,32 +932,31 @@ class QuadObjective(Objective):
         `J(x) = ½ (xᵀ·∇ - xᵀ·b + μ ωᵀ·B·ω)`
         """
         Qx = self.hessp(point)
-        if self.calc_objv:
-            self.lastv = self._value_hessp(point, Qx)
-        return self.hessp(point) - self.data_t
+        self.lastgv = self.value_hessp(point, Qx)
+        return self.hessp(point) - self.hdata_t
 
-    def _value_hessp(self, point, hessp):
-        """Return `J(x)` value given `q = Qx`
+    def value_hessp(self, point, hessp):
+        """Return `J(x)` value at low cost given `x` and `q = Qx`
 
         thanks to relation
 
         `J(x) =  ½ (xᵀ·q - 2 xᵀ·b + μ ωᵀ·B·ω)`"""
         return (
-            # np.sum(point * hessp) - 2 * np.sum(point * self.data_t) + self.constant
-            np.sum(point * (hessp - 2 * self.data_t))
+            np.sum(np.reshape(point, (-1)) * np.reshape(hessp, (-1)))
+            - 2 * np.sum(np.reshape(point, (-1)) * np.reshape(self.hdata_t, (-1)))
             + self.constant
         ) / 2
 
     def value_residual(self, point, residual):
-        """Return `J(x)` value given `x` and `r = b - Qx`
+        """Return `J(x)` value at low cost given `x` and `r = b - Qx`
 
         thanks to relation
 
         `J(x) =  ½ (xᵀ·(-b - r) + μ ωᵀ·B·ω)`"""
-        return (np.sum(point * (-self.data_t - residual)) + self.constant) / 2
+        return (np.sum(point * (-self.hdata_t - residual)) + self.constant) / 2
 
     def norm_mat_major(self, vecs: array, point: array) -> array:
-        return vecs.T @ vecs
+        return np.real(np.conj(vecs.T) @ vecs)
 
     def gr_coeffs(self, point: array) -> array:
         """Return 1."""
@@ -811,7 +979,7 @@ class Vmin(BaseObjective):
         The hyperparameter value `μ`.
     """
 
-    def __init__(self, vmin: float, hyper: float):
+    def __init__(self, vmin: float, hyper: float, name: str = ""):
         """A minimum value objective function
 
         `J(x) = ½ μ ||P_[m, +∞[(x) - m||²`.
@@ -822,8 +990,10 @@ class Vmin(BaseObjective):
             The minimum value `m`.
         hyper : float
             The hyperparameter value `μ`.
+        name: str
+            The name of the objective.
         """
-        super().__init__()
+        super().__init__(name=name)
         self.vmin = vmin
         self.hyper = hyper
 
@@ -836,12 +1006,12 @@ class Vmin(BaseObjective):
 
     def gradient(self, point: array) -> array:
         idx = point <= self.vmin
-        if self.calc_objv:
-            self.lastv = self.hyper * np.sum((point[idx] - self.vmin) ** 2) / 2
+        if self.calc_fun:
+            self.lastgv = self.hyper * np.sum((point[idx] - self.vmin) ** 2) / 2
         return self.hyper * np.where(idx, point - self.vmin, 0)
 
     def norm_mat_major(self, vecs: array, point: array) -> array:
-        return vecs.T @ vecs
+        return np.real(np.conj(vecs.T) @ vecs)
 
 
 class Vmax(BaseObjective):
@@ -857,7 +1027,7 @@ class Vmax(BaseObjective):
         The hyperparameter value `μ`.
     """
 
-    def __init__(self, vmax: float, hyper: float):
+    def __init__(self, vmax: float, hyper: float, name: str = ""):
         """A maximum value objective function
 
         Return `J(x) = ½ μ ||P_[M, +∞[(x) - M||²`.
@@ -868,8 +1038,10 @@ class Vmax(BaseObjective):
             The maximum value `M`.
         hyper : float
             The hyperparameter value `μ`.
+        name: str
+            The name of the objective.
         """
-        super().__init__()
+        super().__init__(name=name)
         self.vmax = vmax
         self.hyper = hyper
 
@@ -882,12 +1054,12 @@ class Vmax(BaseObjective):
 
     def gradient(self, point: array) -> array:
         idx = point >= self.vmax
-        if self.calc_objv:
-            self.lastv = self.hyper * np.sum((point[idx] - self.vmax) ** 2) / 2
+        if self.calc_fun:
+            self.lastgv = self.hyper * np.sum((point[idx] - self.vmax) ** 2) / 2
         return self.hyper * np.where(idx, point - self.vmax, 0)
 
     def norm_mat_major(self, vecs: array, point: array) -> array:
-        return vecs.T @ vecs
+        return np.real(np.conj(vecs.T) @ vecs)
 
 
 class Loss(abc.ABC):
